/************************************************************/
/*                                                          */
/* Module ID  - hcp.                                        */
/*                                                          */
/* Function   - Execute a CP command and display message.   */
/*                                                          */
/* Called By  - N/A.                                        */
/*                                                          */
/* Calling To - N/A.                                        */
/*                                                          */
/* Parameters - (1) Command.                                */
/*              DESCRIPTION: Command to be executed.        */
/*                                                          */
/* Notes      - (1) Uses /dev/cpcmdx to communicate with CP */
/*                                                          */
/*              (2) ....................................... */
/*                                                          */
/*                                                          */
/* Name       - Neale Ferguson.                             */
/*                                                          */
/* Date       - January, 2000.                              */
/*                                                          */
/*                                                          */
/* Associated    - (1) Refer To ........................... */
/* Documentation                                            */
/*                 (2) Refer To ........................... */
/*                                                          */
/************************************************************/
 
/************************************************************/
/*                                                          */
/*                     DEFINES                              */
/*                     -------                              */
/*                                                          */
/************************************************************/
 
#define CPDEV 		"/dev/cpcmd"
#define RSPSIZE 	8192
#define CMDSIZE		512
 
/*=============== End of Defines ===========================*/
 
/************************************************************/
/*                                                          */
/*              INCLUDE STATEMENTS                          */
/*              ------------------                          */
/*                                                          */
/************************************************************/
 
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <linux/ioctl.h>
 
/*================== End of Include Statements =============*/
 
/************************************************************/
/*                                                          */
/*              TYPE DEFINITIONS                            */
/*              ----------------                            */
/*                                                          */
/************************************************************/
 
/* Use 'k' as magic number */
#define CPCMD_IOC_MAGIC  'k'

#define CPCMD_IOCRESET    _IO(CPCMD_IOC_MAGIC, 0)
#define CPIOCRC           _IOR(CPCMD_IOC_MAGIC, 1, int)
 
/*================== End of Type Definitions ===============*/
 
/************************************************************/
/*                                                          */
/*             FUNCTION PROTOTYPES                          */
/*             -------------------                          */
/*                                                          */
/************************************************************/
 
/*================== End of Prototypes =====================*/
 
/************************************************************/
/*                                                          */
/*           GLOBAL VARIABLE DECLARATIONS                   */
/*           ----------------------------                   */
/*                                                          */
/************************************************************/
 
 
/*============== End of Variable Declarations ==============*/
 
/************************************************************/
/*                                                          */
/* Name       - main.                                       */
/*                                                          */
/* Function   - Open the CP command device.                 */
/*                                                          */
/* Parameters - inode -                                     */
/*              filp  -                                     */
/*                                                          */
/************************************************************/
 
int
main(int argc, char **argv)
{
 int fd, rc, l_command,
     i_arg, exitRc = 0;
 char *response,
      *command;
 
    if (argc < 2) {
       exitRc = -1;
       fprintf(stderr, "Correct form: hcp <command>\n");
    }
    else {
       response = malloc(RSPSIZE);
       command  = malloc(CMDSIZE);
       command[0] = 0;
       for (i_arg = 1;
            strlen((const char *) command) < CMDSIZE && i_arg < argc;
            i_arg++) {
          l_command = CMDSIZE - strlen(command) - 2;
          strncat(command, argv[i_arg], l_command);
          l_command += 2;
          strncat(command, " ", l_command);
       }
       fd = open(CPDEV, O_RDWR);
       if (fd >= 0) {
          rc = write(fd, command, strlen(command));
          if (rc >= 0) {
             rc = read(fd, response, RSPSIZE-1);
             if (rc >= 0) {
                response[rc] = 0;
                printf("%s", response);
		rc = ioctl(fd, CPIOCRC, &exitRc);
		if (rc != 0) {
                   exitRc = errno;
                   fprintf(stderr, "Read: %s\n", strerror(errno));
                }
             }
             else {
                exitRc = errno;
                fprintf(stderr, "Read: %s\n", strerror(errno));
             }
          }
          else {
             exitRc = errno;
             fprintf(stderr, "Write: %s\n", strerror(errno));
          }
       }
       else {
          exitRc = errno;
          fprintf(stderr, "Open: %s\n", strerror(errno));
       }
       close(fd);
       free(command);
       free(response);
    }
    exit(exitRc);
}
 
/*===================== End of Function ====================*/
