/************************************************************/
/*                                                          */
/* Module ID  - main.c                                      */
/*                                                          */
/* Function   - Provide interface between Linux/390 (VM) &  */
/*              various CP interfaces.                      */
/*                                                          */
/* Called By  -                                             */
/*                                                          */
/* Calling To -                                             */
/*                                                          */
/* Parameters - (1)                                         */
/*              DESCRIPTION:                                */
/*                                                          */
/* Notes      - (1) ....................................... */
/*                                                          */
/*              (2) ....................................... */
/*                                                          */
/*                                                          */
/* Name       - Neale Ferguson.                             */
/*                                                          */
/* Date       - January, 2000.                              */
/*                                                          */
/*                                                          */
/* Associated    - (1) Refer To ........................... */
/* Documentation                                            */
/*                 (2) Refer To ........................... */
/*                                                          */
/************************************************************/
 
/************************************************************/
/*                                                          */
/*                     DEFINES                              */
/*                     -------                              */
/*                                                          */
/************************************************************/

#ifndef __KERNEL__
#  define __KERNEL__
#endif
#ifndef MODULE
#  define MODULE
#endif
 
#define __NO_VERSION__ /* don't define kernel_version in module.h */
 
/*=============== End of Defines ===========================*/
 
/************************************************************/
/*                                                          */
/*              INCLUDE STATEMENTS                          */
/*              ------------------                          */
/*                                                          */
/************************************************************/
 
#include <linux/version.h>
#include <linux/module.h>
 
#include <linux/kernel.h> 	/* printk() */
#include <linux/slab.h> 	/* kmalloc() */
#include <linux/fs.h>     	/* everything... */
#include <linux/errno.h> 	/* error codes */
#include <linux/types.h>  	/* size_t */
#include <linux/proc_fs.h>
#include <linux/fcntl.h>        /* O_ACCMODE */
 
#include <asm/system.h>   	/* cli(), *_flags */
#include <asm/segment.h>  	/* memcpy and such */
#include <asm/ebcdic.h>   	/* ebcdic stuff */
#include <asm/uaccess.h>  	/* copy to/from user space */
 
#include "cpint.h"        	/* local definitions */
 
/*================== End of Include Statements =============*/
 
/************************************************************/
/*                                                          */
/*              TYPE DEFINITIONS                            */
/*              ----------------                            */
/*                                                          */
/************************************************************/
 
 
/*================== End of Type Definitions ===============*/
 
/************************************************************/
/*                                                          */
/*             FUNCTION PROTOTYPES                          */
/*             -------------------                          */
/*                                                          */
/************************************************************/

MODULE_AUTHOR ("Neale Ferguson <Neale.Ferguson@SoftwareAG-USA.com>");
MODULE_DESCRIPTION ("Linux on S/390 interface to VM CP command."
			" Copyright 2000-2003 Neale Ferguson");
MODULE_LICENSE("GPL");

int cpint_open (struct inode *, struct file *);
int init_module(void);
void cleanup_module(void);
 
/*================== End of Prototypes =====================*/
 
/************************************************************/
/*                                                          */
/*           GLOBAL VARIABLE DECLARATIONS                   */
/*           ----------------------------                   */
/*                                                          */
/************************************************************/
 
int cpint_major   = CPINT_MAJOR;
int cpint_nr_devs = CPINT_NR_DEVS;
 
CPInt_Dev *cpint_devices;
 
/*----------------------------------------------------------*/
/* The different file operations                            */
/*----------------------------------------------------------*/

struct file_operations cpint_fops = {
    open:	cpint_open,
};
 
struct file_operations cpcmd_fops = {
    read:  	cpcmd_read,
    write: 	cpcmd_write,
    open: 	cpcmd_open,
    ioctl:	cpcmd_ioctl,
    release:	cpcmd_release,
};
 
struct file_operations applmon_fops = {
    write:	applmon_write,
    open:	applmon_open,
    ioctl:      applmon_ioctl,
    release:	applmon_release,
};

struct file_operations account_fops = {
    write:	account_write,
    open:	account_open,
    release:	account_release,
};
 
/*----------------------------------------------------------*/
/* Map the minor number to an index into a file-ops table   */
/* - value represents an index into the cpFops structure    */
/*----------------------------------------------------------*/
static int fopMap[256] =
{-1, -1, -1, -1, -1, -1, -1, -1,  0, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  2, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
 
/*----------------------------------------------------------*/
/* Array of file-operations structures for each device type */
/*----------------------------------------------------------*/
struct file_operations *cpFops[] = {
   &cpcmd_fops,
   &applmon_fops,
   &account_fops
};
 
/*============== End of Variable Declarations ==============*/
 
/************************************************************/
/*                                                          */
/* Name       - cpint_open.                                 */
/*                                                          */
/* Function   - Determine what CP interface "device" the    */
/*              caller requires. Check that it exists and   */
/*              if so, install the appropriate file_ops     */
/*              pointers and call the required open routine.*/
/*                                                          */
/* Parameters - inode -                                     */
/*              filp  -                                     */
/*                                                          */
/************************************************************/
 
int
cpint_open (struct inode *inode, struct file *filp)
{
 int num  = MINOR(inode->i_rdev),
     opIndex;
 CPInt_Dev *dev;
 
    if (num >= cpint_nr_devs)
       return -ENODEV;
 
    dev = &cpint_devices[num];
 
    opIndex = fopMap[num];
    if (opIndex > -1) {
       filp->f_op = cpFops[opIndex];
       return(filp->f_op->open(inode, filp));
    }
 
    return -ENODEV;
}
 
/*===================== End of Function ====================*/
 
/************************************************************/
/*                                                          */
/* Name       - init_module.                                */
/*                                                          */
/* Function   - Initialize this device driver.              */
/*                                                          */
/************************************************************/
 
int
init_module(void)
{
 int result,
     i_dev;
 CPInt_Dev *dev;
 union {
     long long cpuid;
     char      cpu[8];
 } id;

    asm ("STIDP\t%0\n"
	 : "=m" (id.cpuid)
	);

    if (id.cpu[0] != 0xff)
        return -EACCES;

    /*------------------------------------------------------*/
    /* Register your major, and accept a dynamic number     */
    /*------------------------------------------------------*/
    result = register_chrdev(cpint_major, "cpint", &cpint_fops);
    if (result < 0) {
        printk(KERN_WARNING "cpint: can't get major %d\n",cpint_major);
        return result;
    }
    if (cpint_major == 0)
       cpint_major = result;
 
    /*------------------------------------------------------*/
    /* allocate the devices -- we can't have them static,   */
    /* as the number can be specified at load time          */
    /*------------------------------------------------------*/
    cpint_devices = kmalloc(cpint_nr_devs * sizeof(CPInt_Dev), GFP_KERNEL);
    if (!cpint_devices) {
        result = -ENOMEM;
        unregister_chrdev(cpint_major, "cpint");
        return result;
    }
 
    memset(cpint_devices, 0, cpint_nr_devs * sizeof (CPInt_Dev));
    for (dev = cpint_devices, i_dev = 0; 
         i_dev < cpint_nr_devs; 
         dev++, i_dev++)
       init_waitqueue_head(&dev->devWait);
 
    return 0;
 
}
 
/*===================== End of Function ====================*/
 
/************************************************************/
/*                                                          */
/* Name       - cleanup_module.                             */
/*                                                          */
/* Function   - Unload this device driver.                  */
/*                                                          */
/************************************************************/
 
void
cleanup_module(void)
{
 int i_dev;
 
    unregister_chrdev(cpint_major, "cpint");
    for (i_dev = 0; i_dev < CPINT_NR_DEVS; i_dev++) {
        if (cpint_devices[i_dev].devExt)
           kfree(cpint_devices[i_dev].devExt);
    }
    kfree(cpint_devices);
 
}
 
/*===================== End of Function ====================*/
